Package: seq.hotSPOT
Type: Package
Title: Targeted sequencing panel design based on mutation hotspots
Version: 1.12.0
Authors@R: c(
  person("Sydney", "Grant", email = "sydney.grant@roswellpark.org", role = c("aut", "cre")),
  person("Lei", "Wei", email = "lei.wei@roswellpark.org", role = c("aut")),
  person("Gyorgy", "Paragh", email = "gyorgy.paragh@roswellpark.org", role = c("aut")))
Description: seq.hotSPOT provides a resource for designing effective
        sequencing panels to help improve mutation capture efficacy for
        ultradeep sequencing projects. Using SNV datasets, this package
        designs custom panels for any tissue of interest and identify
        the genomic regions likely to contain the most mutations.
        Establishing efficient targeted sequencing panels can allow
        researchers to study mutation burden in tissues at high depth
        without the economic burden of whole-exome or whole-genome
        sequencing. This tool was developed to make high-depth
        sequencing panels to study low-frequency clonal mutations in
        clinically normal and cancerous tissues.
License: Artistic-2.0
Encoding: UTF-8
LazyData: FALSE
RoxygenNote: 7.2.3
biocViews: Software, Technology, Sequencing, DNASeq, WholeGenome
Imports: R.utils, hash, stats, base, utils
Suggests: BiocStyle, knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
URL: https://github.com/sydney-grant/seq.hotSPOT
BugReports: https://github.com/sydney-grant/seq.hotSPOT/issues
Config/testthat/edition: 3
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 13:00:32 UTC
RemoteUrl: https://github.com/bioc/seq.hotSPOT
RemoteRef: RELEASE_3_23
RemoteSha: b774b0324109969938a49a8995d56b8a9fdb4c7e
NeedsCompilation: no
Packaged: 2026-05-01 09:39:56 UTC; root
Author: Sydney Grant [aut, cre],
  Lei Wei [aut],
  Gyorgy Paragh [aut]
Maintainer: Sydney Grant <sydney.grant@roswellpark.org>
Depends: R (>= 3.5.0)
Built: R 4.6.0; ; 2026-05-01 09:49:39 UTC; unix
