# Generated by roxygen2: do not edit by hand

export(amp_pool)
export(com_hotspot)
export(fw_hotspot)
importFrom(R.utils,seqToIntervals)
importFrom(hash,hash)
importFrom(hash,make.keys)
importFrom(hash,values)
importFrom(stats,ftable)
importFrom(stats,median)
importFrom(utils,globalVariables)
